package ch.zhaw.catan;

import java.util.HashMap;
import java.util.Random;

public class Siedler {

    public static void main(String[] args) {

        //Spiel erstellen
        Parser parser = new Parser();
        SiedlerGame game = foundingPhase(parser);
        gamePhase(parser, game);
    }

    private static void gamePhase(Parser parser, SiedlerGame game) {
        boolean running = true;
        boolean diceThrown = false;
        while (running){
            Config.Faction currentPlayerFaction = game.getCurrentPlayerFaction();
            parser.displayGameboard(game.getBoard().getTextView()); //todo jedesmal ausgeben? oder nur wenn neuer Spieler oder separater Befehl?
            parser.playerTurn(currentPlayerFaction);
            if(!diceThrown) {
                throwDice(game, parser);
                diceThrown = true;
            }
            parser.displayPlayerInfo(game.getCurruntPlayerResource(), game.getCurrentPlayerWinpoints());
            switch (parser.getAction()) {
                case NEXTPLAYER:
                    if(caseNextPlayer(parser, game)){
                        running = false;
                    } else {
                        diceThrown = false;
                    }
                    break;
                case BUILDSETTLEMENT:
                    caseBuildStructure(parser, game, Config.Structure.SETTLEMENT);
                    break;
                case BUILDCITY:
                    caseBuildStructure(parser, game, Config.Structure.CITY);
                    break;
                case BUILDROAD:
                    caseBuildStructure(parser, game, Config.Structure.ROAD);
                    break;
                case TRADEWITHBANK:
                    caseTradeWithBank(parser, game);
                    break;
                case QUIT:
                    running = false;
                    parser.quit();
                    break;
                default:
                    parser.errorMessage();
            }
        }
    }

    private static boolean caseNextPlayer(Parser parser, SiedlerGame game){
        Config.Faction winner = game.getWinner();
        if(winner == null) {
            game.switchToNextPlayer();
            return false;
        } else {
            parser.displayWinnertext(winner);
            return true;
        }
    }

    private static void caseTradeWithBank(Parser parser, SiedlerGame game) {
        Config.Resource offer = parser.trade(true);
        Config.Resource want = parser.trade(false);
        if(!game.tradeWithBankFourToOne(offer, want)){
            parser.errorMessage();
        }
    }

    private static void caseBuildStructure(Parser parser, SiedlerGame game, Config.Structure structure) {
        parser.giveCoordinatesForStructures(structure);
        boolean successfully = false;
        if(structure == Config.Structure.SETTLEMENT){
            successfully = game.buildSettlement(parser.getPoint());
        } else if(structure == Config.Structure.CITY) {
            successfully = game.buildCity(parser.getPoint());
        } else if(structure == Config.Structure.ROAD){
            successfully = game.buildRoad(parser.getPoint(), parser.getPoint());
        }
        if(!successfully){
            parser.errorMessage();
        }
    }

    private static void throwDice(SiedlerGame game, Parser parser) {
        Random random = new Random();
        //sum of two integers from 0-5 + 2 --> sum of two integers from 1-6
        int thrownDices = random.nextInt(6) + random.nextInt(6) + 2;
        //todo check if 7
        parser.thrownDices(thrownDices);
        game.throwDice(thrownDices);
    }


    private static SiedlerGame foundingPhase(Parser parser) {
        HashMap<String, Integer> gameInfo = parser.gameStart();
        SiedlerGame game = new SiedlerGame(gameInfo.get("NumberOfWinPoints"), gameInfo.get("NumberOfPlayers"));

        for(int player = 1; player <= gameInfo.get("NumberOfPlayers"); player++){
            buildStructuresInFoundingPhase(game, parser, false);
            if(player < gameInfo.get("NumberOfPlayers")){
                game.switchToPreviousPlayer();
            }
        }

        for(int player = 1; player <= gameInfo.get("NumberOfPlayers"); player++){
            buildStructuresInFoundingPhase(game, parser, true);
            game.switchToNextPlayer();
        }
        return game;
    }


    private static void buildStructuresInFoundingPhase(SiedlerGame game, Parser parser, Boolean payout){

        parser.displayGameboard(game.getBoard().getTextView());
        parser.playerTurn(game.getCurrentPlayerFaction());


        //build Settlement
        parser.giveCoordinatesForStructures(Config.Structure.SETTLEMENT);
        boolean sucessful = false;
        do{
            if(game.placeInitialSettlement(parser.getPoint(), payout)) {
                sucessful = true;
            }
            else{
                parser.errorMessage();
            }
        }  while(!sucessful);

        //build Road
        parser.displayGameboard(game.getBoard().getTextView());
        parser.giveCoordinatesForStructures(Config.Structure.ROAD);
        sucessful = false;
        do{
            if(game.placeInitialRoad(parser.getPoint(), parser.getPoint())) {
                sucessful = true;
            }
            else{
                parser.errorMessage();
            }
        }  while(!sucessful);
    }


}
